#!/usr/bin/env python
#coding=utf-8

"""
A module with helper functions used during the Models of Neural System
course (WS09/10).

Robert Schmidt
Bartosz Telenczuk
Bernstein Center for Computational Neuroscience

"""

import numpy as np

def euler1d(f_func, x_init, t_max, time_step):
    r"""Euler method for solving first-order ordinary 
    differential equations (ODEs).

    This function implements the simplest numerical solution to
    fist-order one-dimensional ordinary differential equation. It can
    be used to solve any linear/nonlinear equation of this just by
    defining function `f_func` describing the system.

    Please note that the very basic method is not very accurate and
    may lead to instabilities if the time step `time_step` is too large.
   
   
    Parameters
    ----------

    f_func : function identifier 
        Python function (normal or lambda) describing the right-hand-
        side of the ODE

    x_init : float
        initial condition for t=0

    t_max : float
        the final time of the solution

    time_step : float
        time step

    Returns
    -------

    x : array-like
       the (numerical) solution to the equation

    Example
    -------

    >>> def logistic(x,t, r=0.1):
    ...     return r*x*(1-x)
    >>> x = euler1d(logistic, 0.1, 100, 0.01)
    >>> print x[-1]
    0.999592260595
    """
   
    x_sol = x_init
    res = [x_init]
    for time in np.arange(time_step, t_max, time_step):
        delta_x = f_func(x_sol, time)
        x_sol = x_sol + delta_x*time_step
        res.append(x_sol)
    return np.array(res)


if __name__ == "__main__":
    import doctest
    doctest.testmod()

